""" pytest configuration file. """
import os
import sys
import pytest

import rsa_835_parser

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

current_path = os.path.dirname(os.path.realpath(__file__))

@pytest.fixture
def blue_cross_nc_sample():
    """Blue Cross NC sample"""
    path = current_path + "/test_edi_835_files/blue_cross_nc_sample.txt"
    return rsa_835_parser.parse(path)

@pytest.fixture
def emedny_sample():
    """Emedny sample"""
    path = current_path + "/test_edi_835_files/emedny_sample.txt"
    return rsa_835_parser.parse(path)

@pytest.fixture
def united_healthcare_legacy_sample():
    """United Healthcare Legacy sample"""
    path = current_path + "/test_edi_835_files/united_healthcare_legacy_sample.txt"
    return rsa_835_parser.parse(path)

@pytest.fixture
def all_samples():
    """All samples"""
    path = current_path + "/test_edi_835_files"
    return rsa_835_parser.parse(path)



""" Service Adjustment (CAS) """
from rsa_835_parser.elements.identifier import Identifier
from rsa_835_parser.elements.dollars import Dollars
from rsa_835_parser.elements.adjustment_group_code import AdjustmentGroupCode
from rsa_835_parser.elements.adjustment_reason_code import AdjustmentReasonCode
from rsa_835_parser.segments.utilities import split_segment


class ServiceAdjustment:
    """Service Adjustment (CAS)"""

    identification = "CAS"

    identifier = Identifier()
    group_code = AdjustmentGroupCode()
    reason_code = AdjustmentReasonCode()
    amount = Dollars()

    def __init__(self, segment: str):
        self.segment = segment
        segment = split_segment(segment)

        self.identifier = segment[0]
        self.group_code = segment[1]
        self.reason_code = segment[2]
        self.amount = segment[3]

    def __repr__(self):
        return "\n".join(str(item) for item in self.__dict__.items())


if __name__ == "__main__":
    pass



""" Claims Loop """
# Imports from standard library
from typing import Iterator, Tuple, Optional, List
from warnings import warn

# Local application imports
from rsa_835_parser.segments.claim import Claim as ClaimSegment
from rsa_835_parser.segments.entity import Entity as EntitySegment
from rsa_835_parser.segments.reference import Reference as ReferenceSegment
from rsa_835_parser.segments.date import Date as DateSegment
from rsa_835_parser.segments.amount import Amount as AmountSegment
from rsa_835_parser.segments.utilities import find_identifier
from rsa_835_parser.loops.service import Service as ServiceLoop


class Claim:
    """Claims Loop class"""

    initiating_identifier = ClaimSegment.identification

    terminating_identifiers = [ClaimSegment.identification, "SE"]

    def __init__(
        self,
        claim: ClaimSegment = None,
        entities: List[EntitySegment] = None,
        services: List[ServiceLoop] = None,
        references: List[ReferenceSegment] = None,
        dates: List[DateSegment] = None,
        amount: AmountSegment = None,
    ):
        self.claim = claim
        self.entities = entities if entities else []
        self.services = services if services else []
        self.references = references if references else []
        self.dates = dates if dates else []
        self.amount = amount

    def __repr__(self):
        return "\n".join(str(item) for item in self.__dict__.items())

    @property
    def rendering_provider(self) -> Optional[EntitySegment]:
        """ Rendering provider """
        rendering_provider = [
            e for e in self.entities if e.entity == "rendering provider"
        ]
        assert len(rendering_provider) <= 1

        if len(rendering_provider) == 1:
            return rendering_provider[0]

    @property
    def claim_statement_period_start(self) -> Optional[DateSegment]:
        """ Claim statement period start """
        statement_period_start = [
            d for d in self.dates if d.qualifier == "claim statement period start"
        ]
        assert len(statement_period_start) <= 1

        if len(statement_period_start) == 1:
            return statement_period_start[0]

    @property
    def claim_statement_period_end(self) -> Optional[DateSegment]:
        """ Claim statement period end """
        statement_period_end = [
            d for d in self.dates if d.qualifier == "claim statement period end"
        ]
        assert len(statement_period_end) <= 1

        if len(statement_period_end) == 1:
            return statement_period_end[0]

    @property
    def patient(self) -> EntitySegment:
        """ Patient """
        patient = [e for e in self.entities if e.entity == "patient"]
        assert len(patient) == 1

        return patient[0]
    
    @property
    def claim_number(self) -> str:
        """ Claim Number """
        return self.claim.claim_number

    @classmethod
    def build(
        cls, segment: str, segments: Iterator[str]
    ) -> Tuple["Claim", Optional[Iterator[str]], Optional[str]]:
        """ Build claim loop """
        claim = Claim()
        claim.claim = ClaimSegment(segment)

        segment = segments.__next__()
        while True:
            try:
                if segment is None:
                    segment = segments.__next__()

                identifier = find_identifier(segment)

                if identifier == ServiceLoop.initiating_identifier:
                    service, segment, segments = ServiceLoop.build(segment, segments)
                    claim.services.append(service)

                elif identifier == EntitySegment.identification:
                    entity = EntitySegment(segment)
                    claim.entities.append(entity)
                    segment = None

                elif identifier == ReferenceSegment.identification:
                    reference = ReferenceSegment(segment)
                    claim.references.append(reference)
                    segment = None

                elif identifier == DateSegment.identification:
                    date = DateSegment(segment)
                    claim.dates.append(date)
                    segment = None

                elif identifier == AmountSegment.identification:
                    amount = AmountSegment(segment)
                    claim.amount = amount
                    segment = None

                elif identifier in cls.terminating_identifiers:
                    return claim, segments, segment

                else:
                    segment = None
                    message = f"Identifier: {identifier} not handled in claim loop."
                    warn(message)

            except StopIteration:
                return claim, None, None



"""Service Code"""
from rsa_835_parser.elements import Element
from rsa_835_parser.elements.utilities import split_element

class ServiceCode(Element):
    """Service Code class"""

    def parser(self, value: str) -> str:
        value = split_element(value)
        _, code, *_ = value
        return code



""" Elements module """
from abc import ABC, abstractmethod

class Element(ABC):
    """ Element class """
    def __init__(self):
        self.private_name = None

    def __set_name__(self, owner, name):
        self.private_name = '_' + name

    def __get__(self, obj, obj_type=None):
        return getattr(obj, self.private_name)

    def __set__(self, obj, value):
        value = self.parser(value)
        setattr(obj, self.private_name, value)

    @abstractmethod
    def parser(self, value):
        """ parser method """
        pass

class Code:
    """ Code class """

    def __init__(self, code: str, description: str):
        self.code = code
        self.description = description

    def __str__(self) -> str:
        return str(self.__dict__)



"""	Remark Code (RMR) - 4 characters """
from rsa_835_parser.elements import Element, Code

# https://x12.org/codes/remittance-advice-remark-codes
remark_codes = {
    "N630": "Referral not authorized by attending physician.",
    "N650": "This policy was not in effect for this date of loss. No coverage is available.",
    "M53": "Missing/incomplete/invalid days or units of service.",
    "M15": "Separately billed services/tests have been bundled as they are considered components of the same procedure. Separate payment is not allowed.",
    "M80": "Not covered when performed during the same session/date as a previously processed service for the patient.",
    "M86": "Service denied because payment already made for same/similar procedure within set time frame.",
    "MA130": "Your claim contains incomplete and/or invalid information, and no appeal rights are afforded because the claim is unprocessable. Please submit a new claim with the complete/correct information.",
    "N122": "Add-on code cannot be billed by itself.",
    "N20": "Service not payable with other service rendered on the same date.",
    "N6": "Under FEHB law (U.S.C. 8904(b)), we cannot pay more for covered care than the amount Medicare would have allowed if the patient were enrolled in Medicare Part A and/or Medicare Part B.",
    "N640": "Exceeds number/frequency approved/allowed within time period.",
    "N674": "Not covered unless a pre-requisite procedure/service has been provided.",
    "N702": "Decision based on review of previously adjudicated claims or for claims in process for the same/similar type of services.",
    "N781": "Alert: Patient is a Medicaid/ Qualified Medicare Beneficiary. Review your records for any wrongfully collected deductible. This amount may be billed to a subsequent payer.",
    "N782": "Alert: Patient is a Medicaid/ Qualified Medicare Beneficiary. Review your records for any wrongfully collected coinsurance. This amount may be billed to a subsequent payer.",
    "N807": "Payment adjustment based on the Merit-based Incentive Payment System (MIPS).",
}


class RemarkCode(Element):
    """Remark Code class"""

    def parser(self, value: str) -> Code:
        description = remark_codes.get(value, None)
        return Code(value, description)



""" Entity Code """
from rsa_835_parser.elements import Element

# https://ediacademy.com/blog/x12-n101-entity-identifier-codes/
entity_codes = {
	'QC': 'patient',
	'74': 'insured',
	'82': 'rendering provider',
	'85': 'billing provider'
}


class EntityCode(Element):
    """ Entity Code class """

    def parser(self, value: str) -> str:
        return entity_codes.get(value, value)



""" Dollars """
from typing import Optional

from rsa_835_parser.elements import Element

class Dollars(Element):
    """ Dollars class """

    def parser(self, value: str) -> Optional[float]:
        if value != '':
            return float(value)



""" Service Modifier """
from typing import Optional

from rsa_835_parser.elements import Element
from rsa_835_parser.elements.utilities import split_element

class ServiceModifier(Element):
    """ Service Modifier class"""
    def parser(self, value: str) -> Optional[str]:
        value = split_element(value)
        if len(value) > 2:
            return value[2]



""" Utilities for elements module """
from typing import List
from collections import defaultdict


def split_element(segment: str) -> List[str]:
    """different payers use different characters to delineate sub-elements"""
    delim = _identify_delim(segment)
    return segment.split(delim)


def _identify_delim(segment: str) -> str:
    delim_candidates = ["^", ":", ">", "<"]

    value_counts = defaultdict(int)
    for delim in delim_candidates:
        value_counts[delim] = segment.count(delim)

    delim = max(value_counts, key=value_counts.get)

    return delim



""" Organization segment """
from rsa_835_parser.elements.identifier import Identifier
from rsa_835_parser.elements.organization_type import OrganizationType
from rsa_835_parser.segments.utilities import split_segment

class Organization:
    """ Organization (N1) segment """

    identification = 'N1'

    identifier = Identifier()
    type = OrganizationType()

    def __init__(self, segment: str):
        self.segment = segment
        segment = split_segment(segment)

        self.identifier = segment[0]
        self.type = segment[1]
        self.name = segment[2]
        self.identification_code = int(segment[4]) if len(segment) >= 5 else None

    def __repr__(self):
        return '\n'.join(str(item) for item in self.__dict__.items())


if __name__ == '__main__':
    pass



""" Identifier element """
from typing import Optional

from rsa_835_parser.elements import Element

class Identifier(Element):
    """ Identifier class """

    def __set__(self, obj, value):
        if obj.identification != value:
            raise ValueError('class identifier does not match segment identifier.')

        value = self.parser(value)
        setattr(obj, self.private_name, value)

    def parser(self, value: str) -> Optional[str]:
        return value



""" Financial Information segment """
from rsa_835_parser.elements.identifier import Identifier
from rsa_835_parser.elements.payment_method import PaymentMethod
from rsa_835_parser.elements.dollars import Dollars
from rsa_835_parser.elements.integer import Integer
from rsa_835_parser.elements.date import Date
from rsa_835_parser.segments.utilities import split_segment


class FinancialInformation:
    """Financial Information segment class"""

    identification = "BPR"

    identifier = Identifier()
    amount_paid = Dollars()
    payment_method = PaymentMethod()
    routing_number = Integer()
    transaction_date = Date()

    def __init__(self, segment: str):
        self.segment = segment
        segment = split_segment(segment)

        self.identifier = segment[0]
        self.amount_paid = segment[2]
        self.payment_method = segment[4]
        self.routing_number = segment[13]
        self.transaction_date = segment[16]

    def __repr__(self):
        return "\n".join(str(item) for item in self.__dict__.items())


if __name__ == "__main__":
    pass



""" Date segment """
from rsa_835_parser.elements.identifier import Identifier
from rsa_835_parser.elements.date import Date as DateElement
from rsa_835_parser.elements.date_qualifier import DateQualifier
from rsa_835_parser.segments.utilities import split_segment

class Date:
    """ Date (DTM) segment class """
    identification = 'DTM'

    identifier = Identifier()
    date = DateElement()
    qualifier = DateQualifier()

    def __init__(self, segment: str):
        self.segment = segment
        segment = split_segment(segment)

        self.identifier = segment[0]
        self.qualifier = segment[1]
        self.date = segment[2]

    def __repr__(self):
        return '\n'.join(str(item) for item in self.__dict__.items())


if __name__ == '__main__':
    pass



""" Service Loop """
# Import from standard library
from typing import Tuple, Iterator, Optional, List
from warnings import warn

# Local application imports
from rsa_835_parser.segments.service import Service as ServiceSegment
from rsa_835_parser.segments.claim import Claim as ClaimSegment
from rsa_835_parser.segments.date import Date as DateSegment
from rsa_835_parser.segments.reference import Reference as ReferenceSegment
from rsa_835_parser.segments.amount import Amount as AmountSegment
from rsa_835_parser.segments.service_adjustment import (
    ServiceAdjustment as ServiceAdjustmentSegment,
)
from rsa_835_parser.segments.remark import Remark as RemarkSegment
from rsa_835_parser.segments.utilities import find_identifier
from rsa_835_parser.elements.dollars import Dollars


class Service:
    """Service Loop class"""

    initiating_identifier = ServiceSegment.identification
    terminating_identifiers = [
        ServiceSegment.identification,
        ClaimSegment.identification,
        "SE",
    ]

    def __init__(
        self,
        service: ServiceSegment = None,
        dates: List[DateSegment] = None,
        references: List[ReferenceSegment] = None,
        remarks: List[RemarkSegment] = None,
        amount: AmountSegment = None,
        adjustments: List[ServiceAdjustmentSegment] = None,
    ):
        self.service = service
        self.dates = dates if dates else []
        self.references = references if references else []
        self.remarks = remarks if remarks else []
        self.amount = amount
        self.adjustments = adjustments if adjustments else []

    def __repr__(self):
        return "\n".join(str(item) for item in self.__dict__.items())

    @property
    def allowed_amount(self) -> Optional[Dollars]:
        """Allowed amount"""
        if self.amount:
            if self.amount.qualifier == "allowed - actual":
                return self.amount.amount

    @property
    def service_date(self) -> Optional[DateSegment]:
        """Service date"""
        service_date = [d for d in self.dates if d.qualifier == "service"]
        assert len(service_date) <= 1, f"{self.dates}"

        if len(service_date) == 1:
            return service_date[0]

    @property
    def service_period_start(self) -> Optional[DateSegment]:
        """Service period start"""
        service_period_start = [
            d for d in self.dates if d.qualifier == "service period start"
        ]
        assert len(service_period_start) <= 1, f"{self.dates}"

        if len(service_period_start) == 1:
            return service_period_start[0]
        else:
            return self.service_date

    @property
    def service_period_end(self) -> Optional[DateSegment]:
        """Service period end"""
        service_period_end = [
            d for d in self.dates if d.qualifier == "service period end"
        ]
        assert len(service_period_end) <= 1

        if len(service_period_end) == 1:
            return service_period_end[0]
        else:
            return self.service_date

    @classmethod
    def build(
        cls, segment: str, segments: Iterator[str]
    ) -> Tuple["Service", Optional[str], Optional[Iterator[str]]]:
        """Build service loop"""
        service = Service()
        service.service = ServiceSegment(segment)

        while True:
            try:
                segment = segments.__next__()
                identifier = find_identifier(segment)

                if identifier == DateSegment.identification:
                    date = DateSegment(segment)
                    service.dates.append(date)

                elif identifier == AmountSegment.identification:
                    service.amount = AmountSegment(segment)

                elif identifier == RemarkSegment.identification:
                    remark = RemarkSegment(segment)
                    service.remarks.append(remark)

                elif identifier == ReferenceSegment.identification:
                    reference = ReferenceSegment(segment)
                    service.references.append(reference)

                elif identifier == ServiceAdjustmentSegment.identification:
                    service.adjustments.append(ServiceAdjustmentSegment(segment))

                elif identifier in cls.terminating_identifiers:
                    return service, segment, segments

                else:
                    message = f"Identifier: {identifier} not handled in service loop."
                    warn(message)

            except StopIteration:
                return service, None, None


if __name__ == "__main__":
    pass



""" Remark (LQ) segment """
from rsa_835_parser.elements.identifier import Identifier
from rsa_835_parser.elements.remark_qualifier import RemarkQualifier
from rsa_835_parser.elements.remark_code import RemarkCode
from rsa_835_parser.segments.utilities import split_segment

class Remark:
    """Remark (LQ) segment"""
    identification = 'LQ'

    identifier = Identifier()
    qualifier = RemarkQualifier()
    code = RemarkCode()

    def __init__(self, segment: str):
        self.segment = segment
        segment = split_segment(segment)

        self.identifier = segment[0]
        self.qualifier = segment[1]
        self.code = segment[2]

    def __repr__(self):
        return '\n'.join(str(item) for item in self.__dict__.items())


if __name__ == '__main__':
    pass



""" Reference (REF) segment """
from rsa_835_parser.elements.identifier import Identifier
from rsa_835_parser.elements.reference_qualifier import ReferenceQualifier
from rsa_835_parser.segments.utilities import split_segment


class Reference:
    """Reference (REF) segment"""

    identification = "REF"

    identifier = Identifier()
    qualifier = ReferenceQualifier()

    def __init__(self, segment: str):
        self.segment = segment
        segment = split_segment(segment)

        self.identifier = segment[0]
        self.qualifier = segment[1]
        self.value = segment[2]

    def __repr__(self) -> str:
        return "\n".join(str(item) for item in self.__dict__.items())

    def __str__(self) -> str:
        return f"{self.qualifier}: {self.value}"


if __name__ == "__main__":
    pass



""" Interchange Segment """

# Local Imports
from rsa_835_parser.elements.authorization_information_qualifier import (
    AuthorizationInformationQualifier,
)
from rsa_835_parser.elements.organization import Organization
from rsa_835_parser.elements.date import Date
from rsa_835_parser.elements.identifier import Identifier
from rsa_835_parser.segments.utilities import split_segment

class Interchange:
    """ Interchange (ISA) segment class """
    identification = 'ISA'

    identifier = Identifier()
    authorization_information_qualifier = AuthorizationInformationQualifier()
    sender = Organization()
    receiver = Organization()
    transmission_date = Date()

    def __init__(self, segment: str):
        self.segment = segment
        segment = split_segment(segment)

        self.identifier = segment[0]
        self.authorization_information_qualifier = segment[1]
        self.sender = segment[6]
        self.receiver = segment[8]
        self.transmission_date = segment[9] + segment[10]

    def __repr__(self):
        return '\n'.join(str(item) for item in self.__dict__.items())


if __name__ == '__main__':
    pass



""" Adjustment Group Code """
from rsa_835_parser.elements import Element, Code

# https://x12.org/codes/claim-adjustment-group-codes
adjustment_group_codes = {
	'CR': 'corrections and reversals',
	'OA': 'other adjustment',
	'PR': 'patient responsibility',
	'CO': 'contractual obligation',
	'PI': 'payor initiated reduction',
}


class AdjustmentGroupCode(Element):
    """ Adjustment Group Code class """

    def parser(self, value: str) -> Code:
        description = adjustment_group_codes.get(value, None)
        return Code(value, description)



""" Claim Status """
# Standard Imports
from dataclasses import dataclass
from enum import Enum, auto
from warnings import warn

# Local Imports
from rsa_835_parser.elements import Element


class PayerClassification(Enum):
    """Payer Classification Class"""

    PRIMARY = auto()
    SECONDARY = auto()
    TERTIARY = auto()
    UNSPECIFIED = auto()
    UNKNOWN = auto()

    def __str__(self) -> str:
        return str(self.name).lower()


@dataclass
class Status:
    """
    Attributes:

    - code (:class:`str`): The code provided in the EDI 835 file.
    - description (:class:`str`): The description of the code per `stedi <https://www.stedi.com/edi/x12/segment/CLP>`_.
    - payer_classification (:class:`PayerClassification`)
    """

    code: str
    description: str
    payer_classification: PayerClassification


_REGISTRY = [
    Status("1", "processed as primary", PayerClassification.PRIMARY),
    Status("2", "processed as secondary", PayerClassification.SECONDARY),
    Status("3", "processed as tertiary", PayerClassification.TERTIARY),
    Status("4", "denial", PayerClassification.UNSPECIFIED),
    Status(
        "19",
        "processed as primary, forwarded to additional payer(s)",
        PayerClassification.PRIMARY,
    ),
    Status(
        "20",
        "processed as secondary, forwarded to additional payer(s)",
        PayerClassification.SECONDARY,
    ),
    Status(
        "21",
        "processed as tertiary, forwarded to additional payer(s)",
        PayerClassification.TERTIARY,
    ),
    Status("22", "reversal of previous payment", PayerClassification.UNSPECIFIED),
]


def _lookup_status(code: str) -> Status:
    status = [s for s in _REGISTRY if s.code == code]
    if len(status) == 0:
        warn(
            f"ClaimStatus: Code {code} does not match a status in the edi-835-parser claim status registry."
        )
        return Status("code", "uncategorized", PayerClassification.UNKNOWN)

    return status[0]


class ClaimStatus(Element):
    """Claim Status class"""

    def parser(self, value: str) -> Status:
        return _lookup_status(value)



""" Organization Type """
from rsa_835_parser.elements import Element

organization_types = {
	'PE': 'payee',
	'PR': 'payer',
}


class OrganizationType(Element):
    """ Organization Type class """

    def parser(self, value: str) -> str:
        value = value.strip()
        return organization_types.get(value, value)



""" Setup file for rsa-835-parser """
from setuptools import setup, find_packages

VERSION = "1.40.0"
DESCRIPTION = "B Consulting Group EDI 835 file format parser."
LONG_DESCRIPTION = (
    "B Consulting EDI 835 file format parser, extending the Keiron Stoddart version."
)

install_requires = ["pandas"]

tests_require = ["pytest"]

setup(
    name="rsa-835-parser",
    version=VERSION,
    author="Roberto Suarez",
    author_email="<robsua68@hotmail.com>",
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    packages=find_packages(),
    install_requires=install_requires,
    url="https://github.com/robsua68/bcg-835-parser",
    keywords=["python", "EDI", "835", "X12"],
    classifiers=[
        "Development Status :: 3 - Alpha",
    ],
    tests_require=tests_require,
    python_requires=">=3.11.5",
)



""" rsa-835-parser package """

# Importing standard libraries
import os
from tkinter import E
from typing import List
from warnings import warn

# Local imports
from rsa_835_parser.transaction_set.transaction_set import TransactionSet
from rsa_835_parser.transaction_set.transaction_sets import TransactionSets


def parse(path: str, debug: bool = False) -> TransactionSets:
    """Parses EDI 835 files into a TransactionSets object"""

    if path[0] == "~":
        path = os.path.expanduser(path)

    transaction_sets = []
    if os.path.isdir(path):
        files = _find_edi_835_files(path)
        for file in files:
            file_path = f"{path}/{file}"
            if debug:
                transaction_set = TransactionSet.build(file_path)
                transaction_sets.append(transaction_set)
            else:
                try:
                    transaction_set = TransactionSet.build(file_path)
                    transaction_sets.append(transaction_set)
                except FileNotFoundError as the_error:
                    warn(
                        f"Failed to build transaction_set from {file_path} with error: {the_error}"
                    )
    else:
        transaction_set = TransactionSet.build(path)
        transaction_sets.append(transaction_set)

    return TransactionSets(transaction_sets)


def _find_edi_835_files(path: str) -> List[str]:
    files = []
    for file in os.listdir(path):
        if file.endswith(".txt") or file.endswith(".835"):
            files.append(file)

    return files


def main():
    """Main function"""
    # The original code // Later, try to find the error
    # ---------------------------------------------------
    # data = parse("~/Desktop/eobs").to_dataframe()
    # data.to_csv("~/Desktop/transaction_sets.csv")


if __name__ == "__main__":
    main()



""" Test EDI 835 Parser. """
from tests.conftest import current_path

def test_claim_count(
    blue_cross_nc_sample,
    emedny_sample,
    united_healthcare_legacy_sample,
    all_samples,
):
    """ Test claim count. """
    assert blue_cross_nc_sample.count_claims() == 1
    assert united_healthcare_legacy_sample.count_claims() == 2
    assert emedny_sample.count_claims() == 3
    assert all_samples.count_claims() == 6

def test_patient_count(
    blue_cross_nc_sample,
    emedny_sample,
    united_healthcare_legacy_sample,
    all_samples,
):
    """ Test patient count. """
    assert blue_cross_nc_sample.count_patients() == 1
    assert united_healthcare_legacy_sample.count_patients() == 2
    assert emedny_sample.count_patients() == 3
    assert all_samples.count_patients() == 6

def test_to_dataframe(
    blue_cross_nc_sample,
    emedny_sample,
    united_healthcare_legacy_sample,
    all_samples,
):
    """ Test to_dataframe method."""

    # Blue Cross Example
    payment = blue_cross_nc_sample.sum_payments()
    blue_cross_nc_data = blue_cross_nc_sample.to_dataframe()

    assert payment == blue_cross_nc_data["paid_amount"].sum()

    blue_cross_nc_data.to_csv(f"{current_path}/output/blue_cross_nc_sample.csv")

    # Emedny Example
    payment = emedny_sample.sum_payments()
    emedny_data = emedny_sample.to_dataframe()

    assert payment == emedny_data["paid_amount"].sum()

    emedny_data.to_csv(f"{current_path}/output/emedny_sample.csv")

    # United Healthcare Legacy Example
    payment = united_healthcare_legacy_sample.sum_payments()
    united_healthcare_legacy_ = united_healthcare_legacy_sample.to_dataframe()

    assert payment == united_healthcare_legacy_["paid_amount"].sum()

    united_healthcare_legacy_.to_csv(
        f"{current_path}/output/united_healthcare_legacy_sample.csv"
    )

    # All Samples
    payment = all_samples.sum_payments()
    all_data = all_samples.to_dataframe()

    assert payment == all_data["paid_amount"].sum().round(2)

    all_data.to_csv(f"{current_path}/output/all_samples.csv")



""" Payment Method (PMT) - 3 characters """
from rsa_835_parser.elements import Element

payment_methods = {
	'ACH': 'automatic deposit',
	'CHK': 'check',
	'NON': 'no payment'
}


class PaymentMethod(Element):
    """ Payment Method class """

    def parser(self, value: str) -> str:
        value = value.strip()
        return payment_methods.get(value, value)
